program Hyper;

{$APPTYPE CONSOLE}

uses
  SysUtils;

type
  //    
  TRealPoint = record
    X, Y: Real
  end;
  //    
  TChooseType = (ctRandom, ctUser, ctDistance);

const
  //  
  Count = 10;
  //  
  FileName = 'points.txt';
  //       
  Mes = 'Enter the number that determines how to choose the base points:' + sLineBreak +
    '0 - random,' + sLineBreak +
    '1 - user,' + sLineBreak +
    '2 - minimal distance.' + sLineBreak +
    'Number: ';

var
  I: Integer;
  //  
  Points: array [0..Count - 1] of TRealPoint;
  //    
  ChooseType: TChooseType;
  //  
  A, B: TRealPoint;
  //   
  NA, NB: Integer;
  //       
  CosA, SinA, P: Real;

//     XN(x;y)
// [ N -  ( ),
// x  y -    ,   ";"]
//    TRealPoint.
//  :
//   S - ,  ,
//   N  P -     .
//   True    
//  False   
function StringToRealPoint(const S: string; var N: Integer;
  var P: TRealPoint): Boolean;
var
  L: Integer;           //  
  Semicolon: Integer;   //   ";"  
begin
  L := Length(S);
  Result := L > 0;
  if Result then
    try
      N := StrToInt(S[2]);
      Semicolon := Pos(';', S);
      with P do
      begin
        X := StrToFloat(Copy(S, 4, Semicolon - 4));
        Y := StrToFloat(Copy(S, Semicolon + 1, L - Semicolon - 1))
      end
    except
      Result := False
    end
end;

//    Points,
//       FileName.
//   False   :
// -  /,
// -    ,
// -       Count.
function LoadPointsFromFile: Boolean;
var
  F: TextFile;    // 
  S: string;      //  
  RP: TRealPoint; // 
  I, N: Integer;  //     
begin
  {$I-}
  //    
  AssignFile(F, FileName);
  Reset(F);
  {$I+}
  Result := IOResult = 0;
  //       /
  if not Result then
    Exit;
  //    
  I := 0;
  while (I < Count) and not EOF(F) do
  begin
    ReadLn(F, S);
    Result := StringToRealPoint(S, N, RP);
    //       
    if not Result then
      Break;
    Points[N] := RP;
    Inc(I)
  end;
  //       Count
  Result := I = Count;
  //  
  CloseFile(F)
end;

//       P1  P2
function GetDistance(P1, P2: TRealPoint): Real;
begin
  Result := Sqrt(Sqr(P2.X - P1.X) + Sqr(P2.Y - P1.Y))
end;

//     A  B,
// ..   () 
// A, B, NA, NB,
//   ChooseType.
//  ChooseType = ctUser,  NA  NB  
//      
procedure DetermineBasePoints;
var
  I, J: Integer;
  //       Points[I]  Points[J]
  MinD, D: Real;
begin
  case ChooseType of
    ctRandom:
      begin
        //         0..Count-1
        Randomize;
        NA := Random(Count);
        repeat
          NB := Random(Count)
        until NA <> NB
      end;
    ctDistance:
      begin
        //   ,
        //      0  1
        NA := 0;
        NB := 1;
        MinD := GetDistance(Points[0], Points[1]);
        for I := 0 to Count - 1 do
          for J := I + 1 to Count - 1 do
          begin
            //      
            //   ,
            //   
            // MinD, NA  NB
            D := GetDistance(Points[I], Points[J]);
            if D < MinD then
            begin
              MinD := D;
              NA := I;
              NB := J
            end
          end
      end
  end;
  A := Points[NA];
  B := Points[NB]
end;

//       
//  ,   AB
//     ,
//    CosA * x + SinA * y - P = 0
procedure GetHyperLineEquation;
var
  K, kA, kB, kC, M: Real;
  C: TRealPoint;
begin
  //    AB
  C.X := (A.X + B.X) / 2;
  C.Y := (A.Y + B.Y) / 2;
  //   ,   AB
  K := (A.X - B.X) / (B.Y - A.Y);
  //  ,   AB
  //      ( C),
  //   : kA * x + kB * y + kC = 0
  kA := K;
  kB := -1;
  kC := -K * C.X + C.Y;
  //  
  M := 1 / Sqrt(Sqr(kA) + Sqr(kB));
  //         kC
  if kC * M > 0 then
    M := -1 * M;
  //       
  // CosA * x + SinA * y - P = 0
  CosA := kA * M;
  SinA := kB * M;
  P := -1 * kC * M
end;

begin
  //    
  if LoadPointsFromFile then
  begin
    //     
    Write(Mes);
    ReadLn(I);
    // TChooseType - enumerated type,
    // Ord(ctRandom) = 0, Ord(ctUser) = 1, Ord(ctDistance) = 2
    //  
    ChooseType := TChooseType(I);
    //     ,     
    if ChooseType = ctUser then
    begin
      Write('Enter the base points numbers separated by space: ');
      ReadLn(NA, NB)
    end;
    //   (A, B)   (NA, NB)  
    //      
    DetermineBasePoints;
    //     
    GetHyperLineEquation;
    //    
    WriteLn('Base points numbers: ', NA, ', ', NB);
    WriteLn('Point number':12, 'Class':20);
    for I := Low(Points) to High(Points) do
    begin
      //       
      //   ,     A,  -   B.
      //      
      //         
      if CosA * Points[I].X + SinA * Points[I].Y - P < 0 then
        WriteLn(I:12, 'A (D  < 0)':20)
      else
        WriteLn(I:12, 'B (D >= 0)':20)
    end
  end
  else
    WriteLn('An error occured while reading the file. Exit...');
  ReadLn
end.
